#
# rhevm-setup-plugins -- ovirt engine setup
# Copyright (C) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import gettext
_ = lambda m: gettext.dgettext(message=m, domain='rhevm-setup-plugins')


from otopi import util
from otopi import plugin


from ovirt_engine_setup import constants as osetupcons


@util.export
class Plugin(plugin.PluginBase):

    def __init__(self, context):
        super(Plugin, self).__init__(context=context)

    @plugin.event(
        stage=plugin.Stages.STAGE_MISC,
        after=(
            osetupcons.Stages.DB_SCHEMA,
        ),
    )
    def _virt(self):
        #
        # set package names for guest agent
        #
        self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
            options=(
                {
                    'name': 'AgentAppName',
                    'value': ','.join(
                        (
                            'rhev-agent',
                            'rhevm-guest-agent',
                            'rhevm-guest-agent-common',
                            'RHEV-Agent',
                        )
                    )
                },
            ),
        )

        #
        # core: disable migration of CPU pinned VM's
        #
        # Currently migration of CPU pinned VM's is disabled on downstream,
        # but CpuPinMigrationEnabled is set to true.
        #
        self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
            options=(
                {
                    'name': 'CpuPinMigrationEnabled',
                    'value': 'false',
                },
            ),
        )

        #
        # Support only RHEL based hosts in cluster
        #
        self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
            options=(
                {
                    'name': 'ClusterEmulatedMachines',
                    'value': 'rhel6.2.0',
                    'version': '3.0',
                },
                {
                    'name': 'ClusterEmulatedMachines',
                    'value': 'rhel6.3.0',
                    'version': '3.1',
                },
                {
                    'name': 'ClusterEmulatedMachines',
                    'value': 'rhel6.4.0',
                    'version': '3.2',
                },
                {
                    'name': 'ClusterEmulatedMachines',
                    'value': 'rhel6.5.0',
                    'version': '3.3',
                },
                {
                    'name': 'ClusterEmulatedMachines',
                    'value': 'rhel6.5.0',
                    'version': '3.4',
                },
            ),
        )

        #
        # Remove PPC CPU type from the supported list of the engine.
        # This effectively will render using PPC architecture
        # in the system impossible.
        #
        # re-create the value without ppc64 CPUs
        #
        for version, value in self.environment[
            osetupcons.DBEnv.STATEMENT
        ].getVdcOptionVersions(name='ServerCPUList').items():
            self.environment[
                osetupcons.DBEnv.STATEMENT
            ].updateVdcOptions(
                options=(
                    {
                        'name': 'ServerCPUList',
                        'value': ';'.join(
                            c for c in value.split(';')
                            if c and c.split(':')[4] != 'ppc64'
                        ),
                        'version': version,
                    },
                ),
            )

    @plugin.event(
        stage=plugin.Stages.STAGE_MISC,
        after=(
            osetupcons.Stages.DB_SCHEMA,
        ),
    )
    def _network(self):
        #
        # branding: 'rhevm' management bridge
        #
        self.environment[osetupcons.DBEnv.STATEMENT].execute(
            statement="""
                select RenameManagementNetwork(%(name)s)
            """,
            args=dict(
                name='rhevm',
            ),
        )

        #
        # Allow bridges report for 3.1 cluster level
        #
        # The patch allows to handle bridges reported by 3.1 cluster
        # level agents.
        #
        self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
            options=(
                {
                    'name': 'SupportBridgesReportByVDSM',
                    'value': 'true',
                    'version': '3.1',
                },
            ),
        )

    @plugin.event(
        stage=plugin.Stages.STAGE_MISC,
        after=(
            osetupcons.Stages.DB_SCHEMA,
        ),
    )
    def _storage(self):
        #
        # Cluster with Virt or Gluster support
        #
        # Updating the 'AllowClusterWithVirtGlusterEnabled' value to 'false'.
        # So that the user will be able to create a cluster with
        # either Virt or Gluster support but not both.
        #
        self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
            options=(
                {
                    'name': 'AllowClusterWithVirtGlusterEnabled',
                    'value': 'false',
                },
            ),
        )

        #
        # Enable advanced NFS options
        #
        for v in ('3.1', '3.2', '3.3'):
            self.environment[osetupcons.DBEnv.STATEMENT].updateVdcOptions(
                options=(
                    {
                        'name': 'AdvancedNFSOptionsEnabled',
                        'value': 'true',
                        'version': v,
                    },
                ),
            )


# vim: expandtab tabstop=4 shiftwidth=4
