#
# rhevm-setup-plugins -- ovirt engine setup
# Copyright (C) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Core plugin."""


import os
import shutil
import gettext
_ = lambda m: gettext.dgettext(message=m, domain='rhevm-setup-plugins')


from otopi import util
from otopi import plugin


from ovirt_engine_setup import constants as osetupcons
from ovirt_engine_setup import util as osetuputil


@util.export
class Plugin(plugin.PluginBase):
    """
    3.1 upgrade did not copy all files to pki directory,
    luckily we have a backup, let's hope it was kept
    3.3 assumed that if cert.conf is missing this issue is detected
    for some reasons there are setups with cert.conf without cacert.conf
    without more data it is assumed that was created manually.
    """

    _LEGACY_BACKUP = '/etc/pki/rhevm-old'

    _DATABASE = (
        'database.txt',
        'database.txt.attr',
    )
    _TEMPLATES = (
        'cacert.conf',
        'cert.conf',
        'cacert.template',
        'cert.template',
    )

    def __init__(self, context):
        super(Plugin, self).__init__(context=context)
        self._enabled = False

    @plugin.event(
        stage=plugin.Stages.STAGE_VALIDATION,
    )
    def _validation(self):
        self.logger.debug('Checking legacy PKI upgrade failure')
        if (
            os.path.exists(
                osetupcons.FileLocations.OVIRT_ENGINE_PKI_ENGINE_CA_CERT,
            ) and
            (
                not os.path.exists(
                    os.path.join(
                        osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR,
                        'cert.conf'
                    )
                ) or
                not os.path.exists(
                    os.path.join(
                        osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR,
                        'cacert.conf'
                    )
                )
            )
        ):
            self.logger.debug('Found legacy PKI upgrade failure')

            if not os.path.exists(os.path.join(self._LEGACY_BACKUP, 'ca.pem')):
                raise RuntimeError(
                    _('Legacy (rhevm-3.0) PKI backup is not initialized')
                )
            for f in self._TEMPLATES + self._DATABASE:
                if not os.path.exists(
                    os.path.join(self._LEGACY_BACKUP, f)
                ):
                    raise RuntimeError(
                        _(
                            "Legacy (rhevm-3.0) PKI backup missing "
                            "'{file}'"
                        ).format(
                            file=f,
                        )
                    )
            self._enabled = True

    @plugin.event(
        stage=plugin.Stages.STAGE_MISC,
        condition=lambda self: self._enabled,
        before=(
            osetupcons.Stages.CA_AVAILABLE,
        ),
    )
    def _misc(self):
        for f in self._TEMPLATES:
            dst = os.path.join(osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR, f)
            if not os.path.exists(dst):
                self.logger.debug('replacing %s', dst)
                shutil.copyfile(
                    os.path.join(self._LEGACY_BACKUP, f),
                    os.path.join(
                        osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR,
                        f
                    ),
                )
                os.chmod(dst, 0o644)

        if not os.path.exists(
            os.path.join(
                osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR,
                'database.txt.attr',
            )
        ):
            for f in self._DATABASE:
                dst = os.path.join(
                    osetupcons.FileLocations.OVIRT_ENGINE_PKIDIR,
                    f,
                )
                self.logger.debug('replacing %s', dst)
                shutil.copyfile(
                    os.path.join(self._LEGACY_BACKUP, f),
                    dst,
                )
                os.chmod(dst, 0o644)
                os.chown(
                    dst,
                    osetuputil.getUid(
                        self.environment[osetupcons.SystemEnv.USER_ENGINE]
                    ),
                    osetuputil.getGid(
                        self.environment[osetupcons.SystemEnv.GROUP_ENGINE]
                    ),
                )


# vim: expandtab tabstop=4 shiftwidth=4
