#
# rhevm-setup-plugins -- ovirt engine setup
# Copyright (C) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Reports plugin."""


import os
import configparser
import io
import distutils.version
import gettext
_ = lambda m: gettext.dgettext(message=m, domain='rhevm-setup-plugins')


from otopi import util
from otopi import plugin


from ovirt_engine_setup import constants as osetupcons
from ovirt_engine_setup import rhevm_constants as rhevmcons


@util.export
class Plugin(plugin.PluginBase):
    """Reports plugin."""

    def __init__(self, context):
        super(Plugin, self).__init__(context=context)

    def _parse_legacy_dwh_conf(self, filename):
        config = configparser.ConfigParser()
        config.optionxform = str
        with open(filename) as f:
            config.readfp(io.StringIO(u'[default]' + f.read()))
        return dict(config.items('default'))

    _MINIMAL_ETL = '3.3'
    _KB_32_TO_34_UPG = 'https://access.redhat.com/solutions/1144503'

    @plugin.event(
        stage=plugin.Stages.STAGE_SETUP,
        condition=lambda self: os.path.exists(
            rhevmcons.FileLocations.LEGACY_DWH_CONFIG
        ),
    )
    def _setup_prevent_dwh_32(self):
        legacy = self._parse_legacy_dwh_conf(
            rhevmcons.FileLocations.LEGACY_DWH_CONFIG
        )
        etlVersion = legacy.get('etlVersion')
        if (
            etlVersion and
            distutils.version.LooseVersion(
                etlVersion
            ) < distutils.version.LooseVersion(self._MINIMAL_ETL)
        ):
            self.logger.error(
                _(
                    'An obsolete version of DWH was found set up on this '
                    'system.'
                )
            )
            self.dialog.note(
                _(
                    'Before running engine-setup again, please make sure '
                    'that DWH {min_etl} is set up by running the command '
                    '"rhevm-dwh-setup" from rhevm-dwh {min_etl}.\n'
                    'Please see the following article for more information: '
                    '{kb}'
                ).format(
                    min_etl=self._MINIMAL_ETL,
                    kb=self._KB_32_TO_34_UPG,
                ),
            )
            raise RuntimeError(
                _(
                    'An obsolete version of DWH was found set up on this '
                    'system.'
                )
            )

    @plugin.event(
        stage=plugin.Stages.STAGE_SETUP,
        condition=lambda self: os.path.exists(
            rhevmcons.FileLocations.LEGACY_REPORTS_BUILD_CONF_DEFAULT_DEV
        ),
    )
    def _setup_prevent_reports_32(self):
        self.logger.error(
            _(
                'An obsolete version of Reports was found set up on this '
                'system.'
            )
        )
        self.dialog.note(
            _(
                'Before running engine-setup again, please make sure '
                'that Reports {min_etl} is set up by running the command '
                '"rhevm-reports-setup" from rhevm-reports {min_etl}.\n'
                'Please see the following article for more information: '
                '{kb}'
            ).format(
                min_etl=self._MINIMAL_ETL,
                kb=self._KB_32_TO_34_UPG,
            ),
        )
        raise RuntimeError(
            _(
                'An obsolete version of Reports was found set up on this '
                'system.'
            )
        )

    @plugin.event(
        stage=plugin.Stages.STAGE_CLOSEUP,
        before=(
            osetupcons.Stages.DIALOG_TITLES_E_SUMMARY,
        ),
        after=(
            osetupcons.Stages.DIALOG_TITLES_S_SUMMARY,
        ),
        priority=plugin.Stages.PRIORITY_LAST,
    )
    def _closeup(self):
        # TODO Replace this with http://gerrit.ovirt.org/24865
        packages = []
        if (
            os.path.exists(
                rhevmcons.FileLocations.DWH_INSTALLATION_FOLDER
            ) and
            not self.environment.get(rhevmcons.DWHCoreEnv.ENABLE)
        ):
            packages.append('rhevm-dwh')

        if (
            os.path.exists(
                rhevmcons.FileLocations.REPORTS_INSTALLATION_FOLDER
            ) and
            not self.environment.get(rhevmcons.ReportsCoreEnv.ENABLE)
        ):
            packages.append('rhevm-reports')

        if packages:
            self.logger.info(
                _('DWH and/or Reports were found on this system.')
            )
            self.dialog.note(
                text=_(
                    'To upgrade them, please run the following commands:\n'
                    '# yum update {packages}\n'
                    '# engine-setup'
                ).format(
                    packages=' '.join(packages),
                )
            )


# vim: expandtab tabstop=4 shiftwidth=4
