#
# rhevm-setup-plugins -- ovirt engine setup
# Copyright (C) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Setup Reinstallable plugin."""


import gettext
_ = lambda m: gettext.dgettext(message=m, domain='rhevm-setup-plugins')


from otopi import util
from otopi import plugin
from otopi import miniyum


from ovirt_engine_setup import constants as osetupcons
from ovirt_engine_setup import rhevm_constants as rhevmcons


@util.export
class Plugin(plugin.PluginBase):
    """Setup Reinstallable plugin."""

    def __init__(self, context):
        super(Plugin, self).__init__(context=context)

    def _getSink(self):
        class MyMiniYumSink(self._miniyum.MiniYumSinkBase):
            def __init__(self, log):
                super(MyMiniYumSink, self).__init__()
                self._log = log

            def verbose(self, msg):
                super(MyMiniYumSink, self).verbose(msg)
                self._log.debug('Yum: %s', msg)

            def info(self, msg):
                super(MyMiniYumSink, self).info(msg)
                self._log.info('Yum: %s', msg)

            def error(self, msg):
                super(MyMiniYumSink, self).error(msg)
                self._log.error('Yum: %s', msg)
        return MyMiniYumSink(self.logger)

    def _checkIfPackagesReinstallable(self, packages):
        self.logger.debug('Start %s' % packages)
        reinstallable = True
        installed_packages = []
        myum = self._miniyum.MiniYum(
            sink=self._getSink(),
            disabledPlugins=('versionlock',),
        )
        for package in packages:
            installed = False
            reinstall_available = False
            installed_package = None
            for query in myum.queryPackages(
                patterns=(package,),
                showdups=True,
            ):
                self.logger.debug(
                    'dupes: operation [%s] package %s' % (
                        query['operation'],
                        query['display_name'],
                    )
                )
                if query['operation'] == 'installed':
                    installed = True
                    installed_package = query['display_name']
                if query['operation'] == 'reinstall_available':
                    reinstall_available = True
            if installed and not reinstall_available:
                reinstallable = False
                installed_packages.append(installed_package)

        return installed_packages, reinstallable

    @plugin.event(
        stage=plugin.Stages.STAGE_SETUP,
    )
    def setup(self):
        self._miniyum = miniyum

    @plugin.event(
        stage=plugin.Stages.STAGE_CUSTOMIZATION,
        before=(
            osetupcons.Stages.DIALOG_TITLES_E_PACKAGES,
        ),
        after=(
            osetupcons.Stages.DIALOG_TITLES_S_PACKAGES,
        ),
    )
    def _validation(self):
        def tolist(s):
            return [e.strip() for e in s.split(',')]

        installed_packages, reinstallable = self._checkIfPackagesReinstallable(
            packages=tolist(
                self.environment[osetupcons.RPMDistroEnv.ENGINE_SETUP_PACKAGES]
            )
        )
        if not reinstallable:
            self.logger.error(
                _(
                    '{packages} cannot be found in enabled repositories.\n'
                    'Seems like required channels/repositories are not '
                    'available on the system.'
                ).format(
                    packages=installed_packages,
                )
            )
            raise RuntimeError(
                _(
                    'Setup cannot be found in enabled repositories.'
                )
            )


# vim: expandtab tabstop=4 shiftwidth=4
