#
# rhevm-setup-plugins -- ovirt engine setup
# Copyright (C) 2013-2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""
RedHat support plugin.
"""


import platform
import gettext
_ = lambda m: gettext.dgettext(message=m, domain='rhevm-setup-plugins')


from otopi import util
from otopi import plugin
from otopi import constants as otopicons
from otopi import filetransaction

from ovirt_engine_setup import constants as osetupcons
from ovirt_engine_setup import rhevm_constants as rhevmcons
from ovirt_engine_setup import dialog


@util.export
class Plugin(plugin.PluginBase):
    """
    RedHat support plugin.
    """
    def __init__(self, context):
        super(Plugin, self).__init__(context=context)
        self._distribution = platform.linux_distribution(
            full_distribution_name=0
        )[0]
        self._enabled = False

    @plugin.event(
        stage=plugin.Stages.STAGE_INIT,
    )
    def _init(self):
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.CONFIGURE,
            True
        )
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.PROXY_ENABLED,
            None
        )
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.PROXY,
            None
        )
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.PROXY_PORT,
            None
        )
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.PROXY_USER,
            None
        )
        self.environment.setdefault(
            rhevmcons.RedHatSupportPluginEnv.PROXY_PASSWORD,
            None
        )

    @plugin.event(
        stage=plugin.Stages.STAGE_SETUP,
        condition=lambda self: not self.environment[
            osetupcons.CoreEnv.DEVELOPER_MODE
        ]
    )
    def _setup(self):
        if self._distribution == 'redhat':
            self._enabled = self.environment[
                rhevmcons.RedHatSupportPluginEnv.
                CONFIGURE
            ]
        else:
            self.logger.warning(
                _('Unsupported distribution, Red Hat support plugin disabled')
            )

    @plugin.event(
        stage=plugin.Stages.STAGE_CUSTOMIZATION,
        condition=lambda self: self._enabled,
        before=(
            osetupcons.Stages.DIALOG_TITLES_E_MISC,
        ),
        after=(
            osetupcons.Stages.DIALOG_TITLES_S_MISC,
        ),
    )
    def _customization(self):
        """
        Request proxy parameters and update the corresponding configuration
        file, /etc/ovirt-engine/redhatsupportplugin.conf
        """
        if self.environment[
            rhevmcons.RedHatSupportPluginEnv.PROXY_ENABLED
        ] is None:
            self.environment[
                rhevmcons.RedHatSupportPluginEnv.PROXY_ENABLED
            ] = dialog.queryBoolean(
                dialog=self.dialog,
                name='OVESETUP_RED_HAT_SUPPORT_USE_PROXY',
                note=_(
                    'Would you like transactions from the Red Hat Access '
                    'Plugin sent from the RHEV Manager '
                    'to be brokered through a proxy server? '
                    '(@VALUES@) [@DEFAULT@]: '
                ),
                prompt=True,
                true=_('Yes'),
                false=_('No'),
                default=False,
            )

        if self.environment[
            rhevmcons.RedHatSupportPluginEnv.PROXY_ENABLED
        ]:
            self.environment[
                rhevmcons.RedHatSupportPluginEnv.PROXY
            ] = self.dialog.queryString(
                name='OVESETUP_RED_HAT_SUPPORT_PROXY',
                note=_(
                    'Please enter your proxy server so that the Red Hat '
                    'Access Plug-in can reach access.redhat.com.'
                    '[@DEFAULT@]: '
                ),
                prompt=True,
                default='http://localhost',
            )

            if (
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_PORT
                ] is None
            ):
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_PORT
                ] = self.dialog.queryString(
                    name='OVESETUP_RED_HAT_SUPPORT_PROXY_PORT',
                    note=_(
                        'Please enter the port of your proxy server.'
                        ' [@DEFAULT@]: '
                    ),
                    prompt=True,
                    default='3128',
                )

            if (
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_USER
                ] is None
            ):
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_USER
                ] = self.dialog.queryString(
                    name='OVESETUP_RED_HAT_SUPPORT_PROXY_USER',
                    note=_(
                        'If your proxy server requires a user name, '
                        'please enter it now: '
                    ),
                    prompt=True,
                    default='',
                )

            if (
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_USER
                ] not in (None, '')
                and self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_PASSWORD
                ] is None
            ):
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_PASSWORD
                ] = self.dialog.queryString(
                    name='OVESETUP_RED_HAT_SUPPORT_PROXY_PASSWORD',
                    note=_(
                        'Proxy server password: '
                    ),
                    prompt=True,
                    hidden=True,
                )
            self.environment[otopicons.CoreEnv.LOG_FILTER].append(
                self.environment[
                    rhevmcons.RedHatSupportPluginEnv.PROXY_PASSWORD
                ]
            )

        self.environment[
            rhevmcons.RedHatSupportPluginEnv.CONFIGURE
        ] = False

    @plugin.event(
        stage=plugin.Stages.STAGE_MISC,
        condition=lambda self: self._enabled,
    )
    def _misc(self):
        self.environment[otopicons.CoreEnv.MAIN_TRANSACTION].append(
            filetransaction.FileTransaction(
                name=rhevmcons.FileLocations.REDHAT_SUPPORT_PLUGIN_CONFIG,
                content=[
                    'isProxy={value}'.format(
                        value='true' if self.environment[
                            rhevmcons.RedHatSupportPluginEnv.PROXY_ENABLED
                        ] else 'false'
                    ),
                    'proxyURL={proxy}'.format(
                        proxy=self.environment[
                            rhevmcons.RedHatSupportPluginEnv.PROXY
                        ] or ''
                    ),
                    'proxyPort={port}'.format(
                        port=self.environment[
                            rhevmcons.RedHatSupportPluginEnv.PROXY_PORT
                        ] or ''
                    ),
                    'proxyUser={user}'.format(
                        user=self.environment[
                            rhevmcons.RedHatSupportPluginEnv.PROXY_USER
                        ] or ''
                    ),
                    'proxyPassword={password}'.format(
                        password=self.environment[
                            rhevmcons.RedHatSupportPluginEnv.PROXY_PASSWORD
                        ] or ''
                    ),
                ],
                modifiedList=self.environment[
                    otopicons.CoreEnv.MODIFIED_FILES
                ],
                owner=self.environment[osetupcons.SystemEnv.USER_ENGINE],
                mode=0o600,
                enforcePermissions=True,
            )
        )


# vim: expandtab tabstop=4 shiftwidth=4
